@@ -40,12 +40,11 @@ pub fn deserialize_parameters(
     loader_id: &Pubkey,
     keyed_accounts: &[KeyedAccount],
     buffer: &[u8],
-    skip_ro_deserialization: bool,
 ) -> Result<(), InstructionError> {
     if *loader_id == bpf_loader_deprecated::id() {
-        deserialize_parameters_unaligned(keyed_accounts, buffer, skip_ro_deserialization)
+        deserialize_parameters_unaligned(keyed_accounts, buffer)
     } else {
-        deserialize_parameters_aligned(keyed_accounts, buffer, skip_ro_deserialization)
+        deserialize_parameters_aligned(keyed_accounts, buffer)
     }
 }
 
@@ -127,33 +126,28 @@ pub fn serialize_parameters_unaligned(
 pub fn deserialize_parameters_unaligned(
     keyed_accounts: &[KeyedAccount],
     buffer: &[u8],
-    skip_ro_deserialization: bool,
 ) -> Result<(), InstructionError> {
     let mut start = size_of::<u64>(); // number of accounts
     for (i, keyed_account) in keyed_accounts.iter().enumerate() {
         let (is_dup, _) = is_dup(&keyed_accounts[..i], keyed_account);
         start += 1; // is_dup
         if !is_dup {
-            if keyed_account.is_writable() || !skip_ro_deserialization {
-                start += size_of::<u8>(); // is_signer
-                start += size_of::<u8>(); // is_writable
-                start += size_of::<Pubkey>(); // key
-                keyed_account.try_account_ref_mut()?.lamports =
-                    LittleEndian::read_u64(&buffer[start..]);
-                start += size_of::<u64>() // lamports
+            start += size_of::<u8>(); // is_signer
+            start += size_of::<u8>(); // is_writable
+            start += size_of::<Pubkey>(); // key
+            keyed_account
+                .try_account_ref_mut()?
+                .set_lamports(LittleEndian::read_u64(&buffer[start..]));
+            start += size_of::<u64>() // lamports
                 + size_of::<u64>(); // data length
-                let end = start + keyed_account.data_len()?;
-                keyed_account
-                    .try_account_ref_mut()?
-                    .data_as_mut_slice()
-                    .clone_from_slice(&buffer[start..end]);
-                start += keyed_account.data_len()? // data
+            let end = start + keyed_account.data_len()?;
+            keyed_account
+                .try_account_ref_mut()?
+                .set_data_from_slice(&buffer[start..end]);
+            start += keyed_account.data_len()? // data
                 + size_of::<Pubkey>() // owner
                 + size_of::<u8>() // executable
                 + size_of::<u64>(); // rent_epoch
-            } else {
-                start += get_serialized_account_size_unaligned(keyed_account)?;
-            }
         }
     }
     Ok(())
@@ -231,7 +225,7 @@ pub fn serialize_parameters_aligned(
                 .map_err(|_| InstructionError::InvalidArgument)?;
             v.write_all(&keyed_account.try_account_ref()?.data())
                 .map_err(|_| InstructionError::InvalidArgument)?;
-            v.fill(
+            v.resize(
                 MAX_PERMITTED_DATA_INCREASE
                     + (v.write_index() as *const u8).align_offset(align_of::<u128>()),
                 0,
@@ -253,7 +247,6 @@ pub fn serialize_parameters_aligned(
 pub fn deserialize_parameters_aligned(
     keyed_accounts: &[KeyedAccount],
     buffer: &[u8],
-    skip_ro_deserialization: bool,
 ) -> Result<(), InstructionError> {
     let mut start = size_of::<u64>(); // number of accounts
     for (i, keyed_account) in keyed_accounts.iter().enumerate() {
@@ -261,35 +254,31 @@ pub fn deserialize_parameters_aligned(
         start += size_of::<u8>(); // position
         if is_dup {
             start += 7; // padding to 64-bit aligned
-        } else if keyed_account.is_writable() || !skip_ro_deserialization {
+        } else {
             let mut account = keyed_account.try_account_ref_mut()?;
             start += size_of::<u8>() // is_signer
                 + size_of::<u8>() // is_writable
                 + size_of::<u8>() // executable
                 + 4 // padding to 128-bit aligned
                 + size_of::<Pubkey>(); // key
-            account.owner = Pubkey::new(&buffer[start..start + size_of::<Pubkey>()]);
+            account.copy_into_owner_from_slice(&buffer[start..start + size_of::<Pubkey>()]);
             start += size_of::<Pubkey>(); // owner
-            account.lamports = LittleEndian::read_u64(&buffer[start..]);
+            account.set_lamports(LittleEndian::read_u64(&buffer[start..]));
             start += size_of::<u64>(); // lamports
-            let pre_len = account.data_as_mut_slice().len();
+            let pre_len = account.data().len();
             let post_len = LittleEndian::read_u64(&buffer[start..]) as usize;
             start += size_of::<u64>(); // data length
             let mut data_end = start + pre_len;
             if post_len != pre_len
                 && (post_len.saturating_sub(pre_len)) <= MAX_PERMITTED_DATA_INCREASE
             {
-                account.data.resize(post_len, 0);
                 data_end = start + post_len;
             }
-            account
-                .data_as_mut_slice()
-                .clone_from_slice(&buffer[start..data_end]);
+
+            account.set_data_from_slice(&buffer[start..data_end]);
             start += pre_len + MAX_PERMITTED_DATA_INCREASE; // data
             start += (start as *const u8).align_offset(align_of::<u128>());
             start += size_of::<u64>(); // rent_epoch
-        } else {
-            start += get_serialized_account_size_aligned(keyed_account)?;
         }
     }
     Ok(())
@@ -423,11 +412,11 @@ mod tests {
         for ((account, account_info), key) in accounts.iter().zip(de_accounts).zip(keys.clone()) {
             assert_eq!(key, *account_info.key);
             let account = account.borrow();
-            assert_eq!(account.lamports, account_info.lamports());
-            assert_eq!(&account.data()[..], &account_info.data.borrow()[..]);
-            assert_eq!(&account.owner, account_info.owner);
-            assert_eq!(account.executable, account_info.executable);
-            assert_eq!(account.rent_epoch, account_info.rent_epoch);
+            assert_eq!(account.lamports(), account_info.lamports());
+            assert_eq!(account.data(), &account_info.data.borrow()[..]);
+            assert_eq!(account.owner(), account_info.owner);
+            assert_eq!(account.executable(), account_info.executable);
+            assert_eq!(account.rent_epoch(), account_info.rent_epoch);
 
             assert_eq!(
                 (*account_info.lamports.borrow() as *const u64).align_offset(align_of::<u64>()),
@@ -456,26 +445,15 @@ mod tests {
                 }
             })
             .collect();
-        deserialize_parameters(
-            &bpf_loader::id(),
-            &de_keyed_accounts,
-            serialized.as_slice(),
-            true,
-        )
-        .unwrap();
+        deserialize_parameters(&bpf_loader::id(), &de_keyed_accounts, serialized.as_slice())
+            .unwrap();
         for ((account, de_keyed_account), key) in
             accounts.iter().zip(de_keyed_accounts).zip(keys.clone())
         {
             assert_eq!(key, *de_keyed_account.unsigned_key());
             let account = account.borrow();
-            assert_eq!(account.lamports, de_keyed_account.lamports().unwrap());
-            assert_eq!(
-                &account.data()[..],
-                &de_keyed_account.try_account_ref().unwrap().data[..]
-            );
-            assert_eq!(account.owner, de_keyed_account.owner().unwrap());
-            assert_eq!(account.executable, de_keyed_account.executable().unwrap());
-            assert_eq!(account.rent_epoch, de_keyed_account.rent_epoch().unwrap());
+            assert_eq!(account.executable(), de_keyed_account.executable().unwrap());
+            assert_eq!(account.rent_epoch(), de_keyed_account.rent_epoch().unwrap());
         }
 
         // check serialize_parameters_unaligned
@@ -495,11 +473,11 @@ mod tests {
         for ((account, account_info), key) in accounts.iter().zip(de_accounts).zip(keys.clone()) {
             assert_eq!(key, *account_info.key);
             let account = account.borrow();
-            assert_eq!(account.lamports, account_info.lamports());
-            assert_eq!(&account.data()[..], &account_info.data.borrow()[..]);
-            assert_eq!(&account.owner, account_info.owner);
-            assert_eq!(account.executable, account_info.executable);
-            assert_eq!(account.rent_epoch, account_info.rent_epoch);
+            assert_eq!(account.lamports(), account_info.lamports());
+            assert_eq!(account.data(), &account_info.data.borrow()[..]);
+            assert_eq!(account.owner(), account_info.owner);
+            assert_eq!(account.executable(), account_info.executable);
+            assert_eq!(account.rent_epoch(), account_info.rent_epoch);
         }
 
         let de_accounts = accounts.clone();
@@ -519,7 +497,6 @@ mod tests {
             &bpf_loader_deprecated::id(),
             &de_keyed_accounts,
             serialized.as_slice(),
-            true,
         )
         .unwrap();
         for ((account, de_keyed_account), key) in
@@ -527,14 +504,14 @@ mod tests {
         {
             assert_eq!(key, *de_keyed_account.unsigned_key());
             let account = account.borrow();
-            assert_eq!(account.lamports, de_keyed_account.lamports().unwrap());
+            assert_eq!(account.lamports(), de_keyed_account.lamports().unwrap());
             assert_eq!(
-                &account.data()[..],
-                &de_keyed_account.try_account_ref().unwrap().data[..]
+                account.data(),
+                de_keyed_account.try_account_ref().unwrap().data()
             );
-            assert_eq!(account.owner, de_keyed_account.owner().unwrap());
-            assert_eq!(account.executable, de_keyed_account.executable().unwrap());
-            assert_eq!(account.rent_epoch, de_keyed_account.rent_epoch().unwrap());
+            assert_eq!(*account.owner(), de_keyed_account.owner().unwrap());
+            assert_eq!(account.executable(), de_keyed_account.executable().unwrap());
+            assert_eq!(account.rent_epoch(), de_keyed_account.rent_epoch().unwrap());
         }
     }
 
